/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.jcg.springrestdocs;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;
import org.springframework.context.ApplicationContext;
import org.springframework.restdocs.RestDocumentationContextProvider;
import org.springframework.restdocs.RestDocumentationExtension;
import static org.springframework.restdocs.request.RequestDocumentation.parameterWithName;
import static org.springframework.restdocs.request.RequestDocumentation.queryParameters;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.web.reactive.server.WebTestClient;
import static org.springframework.restdocs.webtestclient.WebTestClientRestDocumentation.document;
import static org.springframework.restdocs.webtestclient.WebTestClientRestDocumentation.documentationConfiguration;

/**
 *
 * @author omozegieaziegbe
 */
@ExtendWith({RestDocumentationExtension.class, SpringExtension.class})
@WebFluxTest(BookController.class)
public class BookControllerWebTestClient {

    @Autowired
    private WebTestClient webTestClient;

    @BeforeEach
    public void setUp(ApplicationContext webApplicationContext, RestDocumentationContextProvider restDocumentation) {
        this.webTestClient = WebTestClient.bindToApplicationContext(webApplicationContext)
                .configureClient()
                .filter(documentationConfiguration(restDocumentation))
                .build();
    }
    
    @Test
    public void testGetBooksByGenreAndAuthor() {
        this.webTestClient.get().uri("/api/books?genre=programming&author=Craig Walls")
                .exchange()
                .expectStatus().isOk()
                .expectBody()
                .consumeWith(document("books",
                        queryParameters(
                                parameterWithName("genre").description("Genre of the book"),
                                parameterWithName("author").description("Author of the book")
                        )
                ));
    }

//    @Test
//    public void testGetAllBooks() {
//        this.webTestClient.get().uri("/api/books")
//                .exchange()
//                .expectStatus().isOk()
//                .expectBody()
//                .consumeWith(document("books"));
//    }

}
